# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand

#' Initialize variable
#'
#' @param list_var Either c() or output of init_variable
#' @param name Variable name
#' @param sd Either numeric value or NA. Use to specify range of effect sizes.
#' @param level Numeric value to specify the number of levels to simulate. Default = 2.
#' @param mu Either a numeric value or a numeric vector (of length = level). Default : 0. Not recommended to modify.
#'
#' @return
#' A list with initialized variables
#' @export
#'
#' @examples
#' init_variable(name = "my_varA", sd = 0.50, level = 200)
init_variable <- function(list_var = c(), name = "myVariable", sd = 0.2, level = 2, mu = 0) {
  
  name <- clean_variable_name(name)
  
  # Only mu specified by user => set level param
  if (is.na(sd)) {
    level <- length(mu)
  }
  
  # Validate inputs
  inputs_checking(list_var, name, mu, sd, level)
  
  if (endsWithDigit(name)) {
    warning("Names ending with digits are not allowed. They will be removed from the variable name.")
    name <- removeDigitsAtEnd(name)
  }
  
  # Initialize new variable
  list_var[[name]] <- fillInVariable(name, mu, sd, level)
  
  return(list_var)
}



#' Check if a string ends with a digit
#'
#' This function checks whether a given string ends with a digit.
#'
#' @param string The input string to be checked
#' @return \code{TRUE} if the string ends with a digit, \code{FALSE} otherwise
#' @export
#' @examples
#' endsWithDigit("abc123")  # Output: TRUE
#' endsWithDigit("xyz")     # Output: FALSE
endsWithDigit <- function(string) {
  lastChar <- substring(string, nchar(string))
  return(grepl("[0-9]", lastChar))
}

#' Remove digits at the end of a string
#'
#' This function removes any digits occurring at the end of a given string.
#'
#' @param string The input string from which digits are to be removed
#' @return The modified string with digits removed from the end
#' @export
#' @examples
#' removeDigitsAtEnd("abc123")  # Output: "abc"
#' removeDigitsAtEnd("xyz")     # Output: "xyz"
removeDigitsAtEnd <- function(string) {
  return(gsub("\\d+$", "", string))
}


#' Check Input Parameters
#'
#' This function checks the validity of the input parameters for initializing a variable.
#' It ensures that the necessary conditions are met for the input parameters.
#'
#' @param list_var List containing the variables to be initialized.
#' @param name Name of the variable.
#' @param mu Mean of the variable.
#' @param sd Standard deviation of the variable (optional).
#' @param level Number of levels for categorical variables.
#' 
#' @return NULL
#' @export
#'
#' @examples
#' inputs_checking(list_var = c(), name = "var1", mu = 0, sd = 1, level = 2)
inputs_checking <- function(list_var, name, mu, sd, level) {
  stopifnot(name != "")
  stopifnot(is.character(name))
  stopifnot(is.numeric(mu))
  stopifnot(is.numeric(sd) | is.na(sd))
  stopifnot(is.numeric(level))
  stopifnot(length(level) == 1)
  stopifnot(level >= 2)

  if (!is.null(list_var)) {
    error_msg <- "Non conformable list_var parameter.\nlist_var must be set as an init_var output or initialized as c()"
    if (!is.list(list_var)) {
      stop(error_msg)
    }
  }

  if (length(mu) > 1) {
    stopifnot(length(mu) == level)
  }

  if (is.na(sd)) {
    if (level != length(mu)) {
      stop("sd was specified as NA. mu should have the same length as the number of levels\n")
    }
  }

  # Check if variable is already initialized
  name_not_in_list_var <- identical(which(already_init_variable(list_var, name)), integer(0))
  if (!name_not_in_list_var) {
    message(paste(name, "is already initialized in list_var.\nIt will be updated", sep = " "))
  }

  return(NULL)
}


#' Check if Variable is Already Initialized
#'
#' This function checks if a variable is already initialized in the variable list.
#'
#' @param list_var A list object representing the variable list.
#' @param new_var_name A character string specifying the name of the new variable.
#'
#' @return TRUE if the variable is already initialized, FALSE otherwise.
#' @export
#'
#' @examples
#' my_list <- list(var1 = 1, var2 = 2, var3 = 3)
#' already_initialized <- already_init_variable(list_var = my_list, new_var_name = "myVariable")
already_init_variable <- function(list_var, new_var_name) {
  if (is.null(list_var)) {
    return(FALSE)
  }
  
  var_names <- names(list_var)
  return(new_var_name %in% var_names)
}

#' Fill in Variable
#'
#' This function fills in a variable with simulated data based on the provided parameters.
#'
#' @param name The name of the variable.
#' @param mu A numeric value or a numeric vector (of length = level) representing the mean.
#' @param sd A numeric value representing the standard deviation, or NA if not applicable.
#' @param level A numeric value specifying the number of levels to simulate.
#'
#' @return A data frame or a list containing the simulated data for the variable.
#' @export
#'
#' @examples
#' variable_data <- fillInVariable(name = "myVariable", mu = c(2, 3), sd = NA, level = 2)
fillInVariable <- function(name, mu, sd, level) {
  
  if (length(mu) > 1 | is.na(sd)) {  # Effects given by user
    level <- length(mu)
    l_labels <- paste(name, 1:level, sep = '')
    l_betaEffects <- mu
    column_names <- c(paste("label", name, sep = "_"), name)
    sub_obj <- build_sub_obj_return_to_user(level, metaData = l_labels,
                                       effectsGivenByUser = l_betaEffects,
                                       column_names)
  } else {
    sub_obj <- as.data.frame(list(mu = mu, sd = sd, level = level))
  }
  
  return(sub_obj)  
}

#' Build Sub Object to Return to User
#'
#' This function builds the sub-object to be returned to the user.
#'
#' @param level A numeric value specifying the number of levels.
#' @param metaData A list of labels.
#' @param effectsGivenByUser A list of effects given by the user.
#' @param col_names A character vector specifying the column names to use.
#' @importFrom utils tail
#'
#' @return A list with the sub-object details.
build_sub_obj_return_to_user <- function(level, metaData, effectsGivenByUser, col_names) {
  sub_obj <- list(level = level)
  data <- cbind(metaData, effectsGivenByUser) %>% as.data.frame()
  colnames(data) <- col_names
  var_name <- utils::tail(col_names, n = 1)
  data[, var_name] <- as.numeric(data[, var_name])
  sub_obj$data <- data
  return(sub_obj)
}


#' Add interaction
#'
#' @param list_var A list of variables (already initialized)
#' @param between_var A vector of variable names to include in the interaction
#' @param sd Either numeric value or NA. Use to specify range of effect sizes. Default 0 for no interaction effects.
#' @param mu Either a numeric value or a numeric vector (of length = level). Default : 0. Not recommended to modify.

#'
#' @return
#' A list with initialized interaction
#' @export
#'
#' @examples
#' init_variable(name = "myvarA", sd = 3, level = 200) %>%
#' init_variable(name = "myvarB", sd = 0.2, level = 2 ) %>%
#' add_interaction(between_var = c("myvarA", "myvarB"), sd = 2)
add_interaction <- function(list_var, between_var, sd = 0, mu = 0) {
  name_interaction <- paste(between_var, collapse = ":")
  check_input2interaction(name_interaction, list_var, between_var, mu, sd)
  
  # Check the number of variables in the interaction
  if (length(between_var) > 3) {
    stop("Cannot initialize an interaction with more than 3 variables.")
  }
  
  interactionCombinations <- getNumberOfCombinationsInInteraction(list_var, between_var)
  list_var$interactions[[name_interaction]] <- fillInInteraction(list_var, between_var, mu, sd, interactionCombinations)
  return(list_var)
}

#' Check input for interaction
#'
#' @param name_interaction String specifying the name of the interaction (example: "varA:varB")
#' @param list_var A list of variables (already initialized)
#' @param between_var A vector of variable names to include in the interaction
#' @param mu Either a numeric value or a numeric vector (of length = level)
#' @param sd Either numeric value or NA
#'
#' @return
#' NULL (throws an error if the input is invalid)
#' @export
check_input2interaction <- function(name_interaction, list_var, between_var, mu, sd) {
  # Check if variables in between_var are declared and initialized
  bool_checkInteractionValidity <- function(between_var, list_var) {
    nb_varInInteraction <- length(unique(between_var))
    stopifnot(nb_varInInteraction > 1)
    existingVar_nb <- getListVar(list_var) %in% between_var %>% sum()
    if (existingVar_nb != nb_varInInteraction) {
      return(FALSE)
    } else {
      return(TRUE)
    }
  }
  
  bool_valid_interaction <- bool_checkInteractionValidity(between_var, list_var)
  if (!bool_valid_interaction) {
    stop("At least one variable in between_var is not declared. Variable not initialized cannot be used in an interaction.")
  }
  
  requestedNumberOfValues <- getNumberOfCombinationsInInteraction(list_var, between_var)
  if (is.na(sd) && requestedNumberOfValues != length(mu)) {
    msg_e <- "sd was specified as NA. mu should have the same length as the possible number of interactions:\n"
    msg_e2 <- paste(requestedNumberOfValues, "interaction values are requested.")
    stop(paste(msg_e, msg_e2))
  }
  
  level <- requestedNumberOfValues
  inputs_checking(list_var$interactions, name_interaction, mu, sd, level)
}

#' Get the number of combinations in an interaction
#'
#' @param list_var A list of variables (already initialized)
#' @param between A vector of variable names to include in the interaction
#'
#' @return
#' The number of combinations in the interaction
#' @export
getNumberOfCombinationsInInteraction <- function(list_var, between) {
  levelInlistVar <- getGivenAttribute(list_var, "level") %>% unlist()
  n_combinations <- prod(levelInlistVar[between]) 
  return(n_combinations)
}

#' Fill in interaction
#'
#' @param list_var A list of variables (already initialized)
#' @param between A vector of variable names to include in the interaction
#' @param mu Either a numeric value or a numeric vector (of length = level)
#' @param sd Either numeric value or NA
#' @param level Number of interactions
#'
#' @return
#' A data frame with the filled-in interaction values
#' @export
fillInInteraction <- function(list_var, between, mu, sd, level) {
  if (length(mu) > 1 || is.na(sd)) {
    l_levels <- getGivenAttribute(list_var, "level") %>% unlist()
    l_levelsOfInterest <- l_levels[between]
    l_labels_varOfInterest <- getLabels(l_variables2labelized = between, l_nb_label = l_levelsOfInterest ) 
    
    grid_combination <- getGridCombination(l_labels_varOfInterest)
    n_combinations <- dim(grid_combination)[1]
    column_names <- c(paste("label", between, sep = "_"), paste(between, collapse = ":"))
    sub_dtf <- build_sub_obj_return_to_user(level = n_combinations,
                                            metaData = grid_combination,
                                            effectsGivenByUser = mu, 
                                            col_names = column_names)
  } else {
    sub_dtf <- list(mu = mu, sd = sd, level = level) %>% as.data.frame()
  }
  
  return(sub_dtf)
}

